@@ -1,31 +1,14 @@
-sNCA = function(x, y, dose=0, adm="Extravascular", dur=0, doseUnit="mg", timeUnit="h", concUnit="ug/L", iAUC="", down="Linear", R2ADJ=0.5, MW=0)
+sNCA = function(x, y, dose=0, adm="Extravascular", dur=0, doseUnit="mg", timeUnit="h", concUnit="ug/L", iAUC="", down="Linear", R2ADJ=0.7, MW=0, Keystring="", excludeDelta=1)
 {
-# Author: Kyun-Seop Bae k@acr.kr
-# Last modification: 2021.6.10
-# Called by: tblNCA
-# Calls: BestSlope, IntAUC, Unit, UT
-# INPUT
-#   x: time or similar vector
-#   y: concentration or similar vector
-#   dose: dose, actually given dose, not per kg or per square meter dose!
-#   adm: administration method, "Extravascular", "Bolus", or "Infusion"
-#   dur: duration of infusion
-#   doseUnit: dose unit (not per kg or per m2)
-#   timeUnit: time unit
-#   concUnit: concentration unit
-#   iAUC: data frame for interval AUC. See the example for the detail
-#   down: trapezoidal downward caculation. "Linear" or "Log"
-#   MW: molecular weight
-# RETURNS
-#   NCA result
-
   if (!(is.numeric(x) & is.numeric(y) & is.numeric(dose) & is.numeric(dur) & is.character(adm) & is.character(down))) stop("Check input types!")
   if (toupper(trimws(adm)) == "INFUSION" & !(dur > 0)) stop("Infusion mode should have dur larger than 0!")
 
   NApoints = is.na(x) | is.na(y)
   x = x[!NApoints]             # remove NA points in x
+  if (any(x[order(x)] != x)) stop("Check if the x is sorted in order!")
   y = y[!NApoints]             # remove NA points in y
   n = length(x)
+  
 
   RetNames1 = c("b0", "CMAX", "CMAXD", "TMAX", "TLAG", "CLST", "CLSTP", "TLST", "LAMZHL", "LAMZ",
              "LAMZLL", "LAMZUL", "LAMZNPT", "CORRXY", "R2", "R2ADJ", "AUCLST", "AUCALL",
@@ -48,35 +31,51 @@ sNCA = function(x, y, dose=0, adm="Extravascular", dur=0, doseUnit="mg", timeUni
   }
 
   Units = Unit(doseUnit=doseUnit, timeUnit=timeUnit, concUnit=concUnit, MW=MW)
-  for (i in 1:length(Res)) Res[i] = Res[i] * Units[names(Res[i]),2]
 
   uY = unique(y)
   if (length(uY) == 1) { # Case of all the same values
     Res["CMAX"] = uY
     if (dose > 0) Res["CMAXD"] = uY / dose
     Res["TMAX"] = x[y==uY][1] # First Tmax
-    if (which(y==uY)[1] > 1) {
+    if (uY == 0) {
+      Res["TLAG"] = NA
+      Res["AUCALL"] = 0
+    } else if (which(y==uY)[1] > 1) {
       Res["TLAG"] = x[which(y==uY) - 1]
     } else {
       Res["TLAG"] = 0
     }
-    Res["CLST"] = uY
-    Res["TLST"] = x[y==uY][1]
+    Res["CLST"] = ifelse(uY == 0, NA, uY)
+    Res["TLST"] = ifelse(uY == 0, NA, x[y == uY][1])
     Res["LAMZNPT"] = 0
     Res["b0"] = uY
 
     if (is.data.frame(iAUC)) { # eg iAUC = data.frame(Name=c("AUC[0-12h]","AUC[0-24h]"), Start=c(0,0), End=c(12,24))
       niAUC = nrow(iAUC)
       if (niAUC > 0) {
-         RetNames = union(RetNames1, as.character(iAUC[,"Name"]))
-         for (i in 1:niAUC) {
-          if (adm == "BOLUS") Res[as.character(iAUC[i,"Name"])] = IntAUC(x2, y2, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
-          else Res[as.character(iAUC[i,"Name"])] = IntAUC(x, y, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
+        RetNames1 = union(RetNames1, as.character(iAUC[,"Name"]))
+        for (i in 1:niAUC) {
+          if (all(y == 0) & min(x, na.rm=T) <= min(0, iAUC[i,"Start"]) & max(x, na.rm=T) >= iAUC[i,"End"]) {
+            Res[as.character(iAUC[i,"Name"])] = 0
+          } else if (toupper(trimws(adm)) == "BOLUS") {
+            if (sum(x == 0) == 0) {
+              x2 = c(0, x)
+              y2 = c(uY, y)
+            }
+            Res[as.character(iAUC[i,"Name"])] = IntAUC(x2, y2, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
+          } else {
+            Res[as.character(iAUC[i,"Name"])] = IntAUC(x, y, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
+          }
+          tRowNames = rownames(Units)
+          Units = rbind(Units, Units["AUCLST",])
+          rownames(Units) = c(tRowNames, iAUC[i, "Name"])
         }
       }
     } else {
       niAUC = 0
     }
+    for (i in 1:length(Res)) Res[i] = Res[i] * Units[names(Res[i]),2]
+
     attr(Res, "units") = c(Units[RetNames1,1], rep(Units["AUCLST",1], niAUC))
 
     return(Res)
@@ -113,12 +112,12 @@ sNCA = function(x, y, dose=0, adm="Extravascular", dur=0, doseUnit="mg", timeUni
     }
   }
 
-  tRes = BestSlope(x1, y1, adm)
+  tRes = BestSlope(x1, y1, adm, excludeDelta=excludeDelta)
   if (R2ADJ > 0) {
     if (tRes["LAMZNPT"] < 2) {
-      tRes = DetSlope(x1, y1)
+      tRes = DetSlope(x1, y1, SubTitle=Keystring)
     } else if (tRes["R2ADJ"] < R2ADJ) {
-      tRes = DetSlope(x1, y1, sel.1=which(x1 == tRes["LAMZLL"]), sel.2=which(x1 == tRes["LAMZUL"]))
+      tRes = DetSlope(x1, y1, SubTitle=Keystring, sel.1=which(x1 == tRes["LAMZLL"]), sel.2=which(x1 == tRes["LAMZUL"]))
     }
   }
 
@@ -179,23 +178,24 @@ sNCA = function(x, y, dose=0, adm="Extravascular", dur=0, doseUnit="mg", timeUni
     Res["VSSP"] = Res["MRTIVIFP"]*Res["CLP"]
   }
 
-#  Units = Unit(doseUnit=doseUnit, timeUnit=timeUnit, concUnit=concUnit, MW=MW)
-#  for (i in 1:length(Res)) Res[i] = Res[i] * Units[names(Res[i]),2]
-
   if (is.data.frame(iAUC)) { # eg iAUC = data.frame(Name=c("AUC[0-12h]","AUC[0-24h]"), Start=c(0,0), End=c(12,24))
     niAUC = nrow(iAUC)
     if (niAUC > 0) {
-       RetNames = union(RetNames1, as.character(iAUC[,"Name"]))
+       RetNames1 = union(RetNames1, as.character(iAUC[,"Name"]))
        for (i in 1:niAUC) {
         if (adm == "BOLUS") Res[as.character(iAUC[i,"Name"])] = IntAUC(x2, y2, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
         else Res[as.character(iAUC[i,"Name"])] = IntAUC(x, y, iAUC[i,"Start"], iAUC[i,"End"], Res, down=down)
+        tRowNames = rownames(Units)
+        Units = rbind(Units, Units["AUCLST",])
+        rownames(Units) = c(tRowNames, iAUC[i, "Name"])
       }
     }
   } else {
     niAUC = 0
   }
 
-  attr(Res, "units") = c(Units[RetNames1,1], rep(Units["AUCLST",1], niAUC))
+  for (i in 1:length(Res)) Res[i] = Res[i] * Units[names(Res[i]),2]
+  attr(Res, "units") = Units[RetNames1,1]
   attr(Res, "UsedPoints") = attr(tRes, "UsedPoints")
   return(Res)
 }
